/*
****************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIAYSTOK 2014                               **
**                                                                        **
**                         Home Control System                            **
**                                                                        **
**                           HC_PROCO4 (RPV)                              **
**                      Wersja z dnia : 2018.10.07                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
****************************************************************************
*/


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>


#define false                          0
#define true                           1

#define LEDPort                        PORTA
#define LEDPortDirection               DDRA

#define StatusLedPosInPort             3
#define RS232RxLedPosInPort            4
#define RS485RxLedPosInPort            5
#define RS232TxLedPosInPort            6
#define RS485TxLedPosInPort            7


#define TimerCounterDelayLimit         3000  // 2048
#define StandardLEDPulseDelay          244   // 832

#define RxTxPort                       PORTD
#define RxTxDirectionPort              DDRD
#define RxTxDirPin                     4              // PORTD . 4

#define RTSPin                         6
#define CTSPin                         5
#define ModemControlPort               PORTD
#define ModemStatusPort                PIND
#define ModemDirPort                   DDRD

#define null                           0
#define cr                             0x0D
#define Space                          0x20
#define PartSeparator                  ':'
#define ParamSeparator                 '.'

#define nop() __asm__ __volatile__ ("nop")

// UART BAUD Rate Generator fosc=7.3728 MHz
#define Speed9600bps                   47
#define Speed19200bps                  23
#define Speed38400bps                  11
#define Speed115200bps                 3

#define IdleState                      0
#define BusyState                      1

#define TXDBuffSize                    128
#define RXDBuffSize                    16

typedef struct { uint8_t CycFr ;
                 uint8_t CycTo ;
                 uint8_t State ;
                 uint8_t Buffer [ TXDBuffSize ] ;
               } TXDRecT ;
typedef struct { uint8_t CycFr ;
                 uint8_t CycTo ;
                 uint8_t Buffer [ RXDBuffSize ] ;
               } RXDRecT ;


#define DeviceIDSize                   8
typedef uint8_t DeviceIDType [ DeviceIDSize + 1 ] ;

#define CommandBufferSize              64
typedef uint8_t CommandBufferType [ CommandBufferSize + 1 ] ;

#define ParamBufferSize                32
typedef uint8_t ParamBufferType [ ParamBufferSize + 1 ] ;

typedef void ( * OperationProcT ) ( void ) ;

typedef struct {
                 OperationProcT            SerialService ;
                 CommandBufferType         CommandText ;
                 ParamBufferType           ParamText ;
                 DeviceIDType              DestinationDeviceName ;
                 DeviceIDType              SourceDeviceName ;
                 uint8_t                   CommandTextIndex ;
                 uint8_t                   ParamTextIndex ;
                 uint8_t                   DestDeviceNameIndex ;
                 uint8_t                   SrcDeviceNameIndex ;
                 uint16_t                  SerialLEDDelay ;
               } ServiceType ;

typedef enum { StatusLED ,
               RS232RxLED ,
               RS485RxLED ,
               RS232TxLED ,
               RS485TxLED ,
             } LEDSinalT ;


static ServiceType SerialService ;
volatile static uint8_t TimeEvent ;
static volatile TXDRecT RS232TXDRec ;
static volatile RXDRecT RS232RXDRec ;
static volatile TXDRecT RS485TXDRec ;
static volatile RXDRecT RS485RXDRec ;
static uint16_t RS232RxSerialLEDDelay ;
static uint16_t RS232TxSerialLEDDelay ;
static uint16_t RS485RxSerialLEDDelay ;
static uint16_t RS485TxSerialLEDDelay ;
static uint16_t TimerCounter ;
static uint16_t RS232CommandIndex ;
static uint16_t RS485CommandIndex ;


static uint8_t DeviceName [ ] PROGMEM               = "\r\nHC-PROCO4 - procesor komunikacyjny (filtr pakietow RPV)\r\n" ;
static uint8_t RPVCommand [ ] PROGMEM               = "RPV" ;



static void LEDOn ( LEDSinalT LEDSignal )
{
  /*----------------------------------------------------------------*/
  switch ( LEDSignal )
  {
    case StatusLED  :
      LEDPort &= ~ ( 1 << StatusLedPosInPort ) ;
      break ;
    case RS232RxLED :
      LEDPort &= ~ ( 1 << RS232RxLedPosInPort ) ;
      break ;
    case RS485RxLED :
      LEDPort &= ~ ( 1 << RS485RxLedPosInPort ) ;
      break ;
    case RS232TxLED :
      LEDPort &= ~ ( 1 << RS232TxLedPosInPort ) ;
      break ;
    case RS485TxLED :
      LEDPort &= ~ ( 1 << RS485TxLedPosInPort ) ;
      break ;
  } /* switch */ ;
} /* LEDOn */



static void LEDOff ( LEDSinalT LEDSignal )
{
  /*----------------------------------------------------------------*/
  switch ( LEDSignal )
  {
    case StatusLED  :
      LEDPort &= ( 1 << StatusLedPosInPort ) ;
      break ;
    case RS232RxLED :
      LEDPort |= ( 1 << RS232RxLedPosInPort ) ;
      break ;
    case RS485RxLED :
      LEDPort |= ( 1 << RS485RxLedPosInPort ) ;
      break ;
    case RS232TxLED :
      LEDPort |= ( 1 << RS232TxLedPosInPort ) ;
      break ;
    case RS485TxLED :
      LEDPort |= ( 1 << RS485TxLedPosInPort ) ;
      break ;
  } /* switch */ ;
} /* LEDOff */


static void StartUART232RxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOn ( RS232RxLED ) ;
  RS232RxSerialLEDDelay = StandardLEDPulseDelay ;
} /* StartUART232RxLED */



static void StartUART232TxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOn ( RS232TxLED ) ;
  RS232TxSerialLEDDelay = StandardLEDPulseDelay ;
} /* StartUART232TxLED */


static void StopUART232RxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOff ( RS232RxLED ) ;
} /* StopUART232RxLED */


static void StopUART232TxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOff ( RS232TxLED ) ;
} /* StopUART232TxLED */


static void StartUART485RxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOn ( RS485RxLED ) ;
  RS485RxSerialLEDDelay = StandardLEDPulseDelay ;
} /* StartUART485RxLED */


static void StartUART485TxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOn ( RS485TxLED ) ;
  RS485TxSerialLEDDelay = StandardLEDPulseDelay ;
} /* StartUART485TxLED */


static void StopUART485RxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOff ( RS485RxLED ) ;
} /* StopUART485RxLED */


static void StopUART485TxLED ( void )
{
  /*----------------------------------------------------------------*/
  LEDOff ( RS485TxLED ) ;
} /* StopUART485TxLED */


ISR ( TIMER0_OVF_vect )
{
  /*----------------------------------------------------------------*/
  TimerCounter ++ ;
  if ( TimerCounter > TimerCounterDelayLimit )
  {
    TimerCounter = 0 ;
    TimeEvent = true ;
  } /* if */ ;
  if ( RS232RxSerialLEDDelay )
  {
    RS232RxSerialLEDDelay -- ;
    if ( ! RS232RxSerialLEDDelay )
    {
      StopUART232RxLED ( ) ;
    } /* if */ ;
  } /* if */ ;
  if ( RS232TxSerialLEDDelay )
  {
    RS232TxSerialLEDDelay -- ;
    if ( ! RS232TxSerialLEDDelay )
    {
      StopUART232TxLED ( ) ;
    } /* if */ ;
  } /* if */ ;
  if ( RS485RxSerialLEDDelay )
  {
    RS485RxSerialLEDDelay -- ;
    if ( ! RS485RxSerialLEDDelay )
    {
      StopUART485RxLED ( ) ;
    } /* if */ ;
  } /* if */ ;
  if ( RS485TxSerialLEDDelay )
  {
    RS485TxSerialLEDDelay -- ;
    if ( ! RS485TxSerialLEDDelay )
    {
      StopUART485TxLED ( ) ;
    } /* if */ ;
  } /* if */ ;
} /* TIMER0_OVF_vect */


static void UART485ReceiveMode ( void )
{
  /*----------------------------------------------------------------*/
  RxTxPort &= ~ ( 1 << RxTxDirPin ) ;
} /* UART485ReceiveMode */


static void UART485TransmitMode ( void )
{
  /*----------------------------------------------------------------*/
  RxTxPort |= ( 1 << RxTxDirPin ) ;
} /* UART485TransmitMode */


ISR ( USART0_RX_vect )
{
  uint8_t RecCh ;
  /*----------------------------------------------------------------*/
  RecCh = UDR0 ;
  RS232RXDRec . Buffer [ RS232RXDRec . CycTo ] = RecCh ;
  RS232RXDRec . CycTo ++ ;
  if ( RS232RXDRec . CycTo >= RXDBuffSize )
    RS232RXDRec . CycTo = 0 ;
  StartUART232RxLED ( ) ;
} /* USART0_RX_vect */


ISR ( USART0_TX_vect )
{
  /*----------------------------------------------------------------*/
  if ( RS232TXDRec . CycFr == RS232TXDRec . CycTo )
  {
    RS232TXDRec . State = IdleState ;
  } /* if ... */
  else
  {
    UDR0 = RS232TXDRec . Buffer [ RS232TXDRec . CycFr ] ;
    RS232TXDRec . CycFr ++ ;
    if ( RS232TXDRec . CycFr >= TXDBuffSize )
      RS232TXDRec . CycFr = 0 ;
    StartUART232TxLED ( ) ;
  } /* if ... else */ ;
} /* USART0_TX_vect */


ISR ( USART1_RX_vect )
{
  uint8_t RecCh ;
  /*----------------------------------------------------------------*/
  RecCh = UDR1 ;
  RS485RXDRec . Buffer [ RS485RXDRec . CycTo ] = RecCh ;
  RS485RXDRec . CycTo ++ ;
  if ( RS485RXDRec . CycTo >= RXDBuffSize )
    RS485RXDRec . CycTo = 0 ;
  StartUART485RxLED ( ) ;
} /* USART1_RX_vect */


ISR ( USART1_TX_vect )
{
  /*----------------------------------------------------------------*/
  if ( RS485TXDRec . CycFr == RS485TXDRec . CycTo )
  {
    RS485TXDRec . State = IdleState ;
    UART485ReceiveMode ( ) ;
  } /* if ... */
  else
  {
    UDR1 = RS485TXDRec . Buffer [ RS485TXDRec . CycFr ] ;
    RS485TXDRec . CycFr ++ ;
    if ( RS485TXDRec . CycFr >= TXDBuffSize )
      RS485TXDRec . CycFr = 0 ;
    StartUART485TxLED ( ) ;
  } /* if ... else */ ;
} /* USART1_TX_vect */


static void UART232SendSerial ( uint8_t SerialCh )
{
  uint8_t NextInx ;
  /*----------------------------------------------------------------*/
  cli ( ) ;
  StartUART232TxLED ( ) ;
  if ( RS232TXDRec . State == IdleState )
  {
    RS232TXDRec . State = BusyState ;
    UDR0 = SerialCh ;
    sei ( ) ;
  } /* if ... */
  else
  {
    for ( ; ; )
    {
      NextInx = RS232TXDRec . CycTo + 1 ;
      if ( NextInx >= TXDBuffSize )
        NextInx = 0 ;
      if ( NextInx != RS232TXDRec . CycFr )
        break ;
      sei ( ) ;
      nop ( ) ;
      nop ( ) ;
      nop ( ) ;
      wdt_reset ( ) ;
      cli ( ) ;
    } /* for */ ;
    RS232TXDRec . Buffer [ RS232TXDRec . CycTo ] = SerialCh ;
    RS232TXDRec . CycTo ++ ;
    if ( RS232TXDRec . CycTo >= TXDBuffSize )
      RS232TXDRec . CycTo = 0 ;
    sei ( ) ;
  } /* if ... else */ ;
} /* UART232SendSerial */


static void UART232SendSerialTxt ( uint8_t * String )
{
  uint8_t Data ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Data = * String ++ ;
    if ( Data )
      UART232SendSerial ( Data ) ;
    else
      break ;
  } /* for */ ;
} /* UART232SendSerialTxt */


static void UART232SendSerialFlashTxt ( uint16_t String )
{
  uint8_t Data ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Data = pgm_read_byte_near ( String ) ;
    if ( Data )
      UART232SendSerial ( Data ) ;
    else
      break ;
    String ++ ;
  } /* for */ ;
} /* UART232SendSerialFlashTxt */


static uint8_t UART232DataPresent ( void )
{
  /*----------------------------------------------------------------*/
  if ( RS232RXDRec . CycFr == RS232RXDRec . CycTo )
    return ( false ) ;
  else
    return ( true ) ;
} /* UART232DataPresent */


static uint8_t UART232GetSerial ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  Ch = RS232RXDRec . Buffer [ RS232RXDRec . CycFr ] ;
  RS232RXDRec . CycFr ++ ;
  if ( RS232RXDRec . CycFr >= RXDBuffSize )
    RS232RXDRec . CycFr = 0 ;
  return ( Ch ) ;
} /* UART232GetSerial */


static uint8_t UART485DataPresent ( void )
{
  /*----------------------------------------------------------------*/
  if ( RS485RXDRec . CycFr == RS485RXDRec . CycTo )
    return ( false ) ;
  else
    return ( true ) ;
} /* UART485DataPresent */


static uint8_t UART485GetSerial ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  Ch = RS485RXDRec . Buffer [ RS485RXDRec . CycFr ] ;
  RS485RXDRec . CycFr ++ ;
  if ( RS485RXDRec . CycFr >= RXDBuffSize )
    RS485RXDRec . CycFr = 0 ;
  return ( Ch ) ;
} /* UART485DataPresent */


static void UART0HardwareInit ( uint16_t RS232SerialSpeed )
{
  /*----------------------------------------------------------------*/
  UBRR0H = ( uint8_t ) ( RS232SerialSpeed >> 8 ) ;
  UBRR0L = ( uint8_t ) ( RS232SerialSpeed & 0xFF ) ;
  UCSR0A = 0 ;
  UCSR0B = ( 1 << RXCIE0 ) | ( 1 << TXCIE0 ) | ( 1 << RXEN0 ) | ( 1 << TXEN0 ) ;
  UCSR0C = ( 1 << UCSZ01 ) | ( 1 << UCSZ00 ) ;
} /* USART0HardwareInit */


static void UART1HardwareInit ( uint16_t RS485SerialSpeed )
{
  /*----------------------------------------------------------------*/
  UBRR1H = ( uint8_t ) ( RS485SerialSpeed >> 8 ) ;
  UBRR1L = ( uint8_t ) ( RS485SerialSpeed & 0xFF ) ;
  UCSR1A = 0 ;
  UCSR1B = ( 1 << RXCIE1 ) | ( 1 << TXCIE1 ) | ( 1 << RXEN1 ) | ( 1 << TXEN1 ) ;
  UCSR1C = ( 1 << UCSZ11 ) | ( 1 << UCSZ10 ) ;
} /* UART1HardwareInit */


static void HardwareInit ( uint16_t RS485SerialSpeed ,
                           uint16_t RS232SerialSpeed )
{
  /*----------------------------------------------------------------*/
  TIMSK0 = ( 1 << TOIE0 ) ;
  TCCR0A = 0 ;
  TCCR0B = ( 1 << CS01 ) ; // CS02=0 CS01=1 CS00=0 : F clk div 8
  TCNT0 = 0 ;
  UART0HardwareInit ( RS232SerialSpeed ) ;
  UART1HardwareInit ( RS485SerialSpeed ) ;
} /* HardwareInit */


static void EnvirInit ( void )
{
  uint8_t Bits ;
  /*----------------------------------------------------------------*/
  Bits = ( 1 << StatusLedPosInPort ) | ( 1 << RS232RxLedPosInPort ) |
         ( 1 << RS485RxLedPosInPort ) | ( 1 << RS232TxLedPosInPort ) |
         ( 1 << RS485TxLedPosInPort ) ;
  LEDPortDirection |= Bits ;
  LEDPort |= Bits ;
  Bits = ( 1 << RxTxDirPin ) ;
  RxTxDirectionPort |= Bits ;
  UART485TransmitMode ( ) ;
  ModemDirPort |= ( 1 << CTSPin ) ;
  ModemDirPort &= ~ ( ( 1 << RTSPin ) ) ;
  ModemControlPort |= ( 1 << CTSPin ) ;
} /* EnvirInit */


static void ClearBlock ( void *   Destination ,
                         uint16_t Size )
{
  uint16_t Loop ;
  uint8_t * Dst ;
  /*----------------------------------------------------------------*/
  Dst = ( uint8_t * ) Destination ;
  for ( Loop = 0 ; Loop < Size ; Loop ++ )
    * Dst ++ = null ;
} /* ClearBlock */


static void UART232ProcessData ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
} /* UART232ProcessData */


static uint8_t CompareFlash ( uint8_t * Text ,
                              uint16_t  Pattern )
{
  uint8_t Ch1 ;
  uint8_t Ch2 ;
  /*----------------------------------------------------------------*/
  for ( ; ; )
  {
    Ch1 = * Text ++ ;
    Ch2 = pgm_read_byte_near ( Pattern ++ ) ;
    if ( Ch1 != Ch2 )
    {
      return ( false ) ;
    } /* if */ ;
    if ( Ch1 == 0 )
    {
      return ( true ) ;
    } /* if */ ;
  } /* for */ ;
} /* CompareFlash */


static void ExecuteCommand ( void )
{
  /*----------------------------------------------------------------*/
  if ( CompareFlash ( SerialService . CommandText , ( uint16_t ) RPVCommand ) )
  {
    UART232SendSerialTxt ( SerialService . DestinationDeviceName ) ;
    UART232SendSerial ( PartSeparator ) ;
    UART232SendSerialTxt ( SerialService . SourceDeviceName ) ;
    UART232SendSerial ( PartSeparator ) ;
    UART232SendSerialTxt ( SerialService . CommandText ) ;
    UART232SendSerial ( ParamSeparator ) ;
    UART232SendSerialTxt ( SerialService . ParamText ) ;
    UART232SendSerial ( 0x0D ) ;
  } /* if */ ;
} /* ExecuteCommand */


static void ResetOperation ( void ) ;

static void DestinationIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . DestDeviceNameIndex < DeviceIDSize )
  {
    SerialService . DestinationDeviceName [ SerialService . DestDeviceNameIndex ++ ] = Ch ;
  } /* if */ ;
} /* DestinationIDPutCh */


static void SourceIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . SrcDeviceNameIndex < DeviceIDSize )
  {
    SerialService . SourceDeviceName [ SerialService . SrcDeviceNameIndex ++ ] = Ch ;
  } /* if */ ;
} /* SourceIDPutCh */


static void CommandIDPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . CommandTextIndex < CommandBufferSize )
  {
    SerialService . CommandText [ SerialService . CommandTextIndex ++ ] = Ch ;
  } /* if */ ;
} /* CommandIDPutCh */


static void ParamPutCh ( uint8_t Ch )
{
  /*----------------------------------------------------------------*/
  if ( SerialService . ParamTextIndex < ParamBufferSize )
  {
    SerialService . ParamText [ SerialService . ParamTextIndex ++ ] = Ch ;
  } /* if */ ;
} /* ParamPutCh */


static void SelectParam ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( UART485DataPresent ( ) )
  {
    Ch = UART485GetSerial ( ) ;
    if ( Ch == cr )
    {
      ParamPutCh ( null ) ;
      ExecuteCommand ( ) ;
      ResetOperation ( ) ;
      return ;
    } /* if */;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    ParamPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectParam */


static void SelectCommand ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( UART485DataPresent ( ) )
  {
    Ch = UART485GetSerial ( ) ;
    if ( Ch == cr )
    {
      CommandIDPutCh ( null ) ;
      ExecuteCommand ( ) ;
      ResetOperation ( ) ;
      return ;
    } /* if */;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == ParamSeparator )
    {
      CommandIDPutCh ( null ) ;
      SerialService . SerialService = SelectParam ;
      return ;
    } /* if */;
    CommandIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectCommand */


static void SelectSourceService ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( UART485DataPresent ( ) )
  {
    Ch = UART485GetSerial ( ) ;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == PartSeparator )
    {
      SourceIDPutCh ( null ) ;
      SerialService . SerialService = SelectCommand ;
      return ;
    } /* if */;
    SourceIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectSourceService */


static void SelectDestOperation ( void )
{
  uint8_t Ch ;
  /*----------------------------------------------------------------*/
  if ( UART485DataPresent ( ) )
  {
    Ch = UART485GetSerial ( ) ;
    if ( Ch < Space )
    {
      ResetOperation ( ) ;
      return ;
    } /* if */ ;
    if ( Ch == PartSeparator )
    {
      DestinationIDPutCh ( null ) ;
      SerialService . SerialService = SelectSourceService ;
      return ;
    } /* if */;
    DestinationIDPutCh ( Ch ) ;
  } /* if */ ;
} /* SelectDestOperation */


static void ResetOperation ( void )
{
  /*----------------------------------------------------------------*/
  ClearBlock ( SerialService . CommandText , sizeof ( CommandBufferType ) ) ;
  ClearBlock ( SerialService . ParamText , sizeof ( ParamBufferType ) ) ;
  ClearBlock ( SerialService . DestinationDeviceName , sizeof ( DeviceIDType ) ) ;
  ClearBlock ( SerialService . SourceDeviceName , sizeof ( DeviceIDType ) ) ;
  SerialService . CommandTextIndex = 0 ;
  SerialService . ParamTextIndex = 0 ;
  SerialService . DestDeviceNameIndex = 0 ;
  SerialService . SrcDeviceNameIndex = 0 ;
  SerialService . SerialService = SelectDestOperation ;
} /* ResetOperation */


static void SoftwareInit ( void )
{
  /*----------------------------------------------------------------*/
  RS485RxSerialLEDDelay = 0 ;
  RS485TxSerialLEDDelay = 0 ;
  RS232RxSerialLEDDelay = 0 ;
  RS232TxSerialLEDDelay = 0 ;
  RS232RXDRec . CycFr = 0 ;
  RS232RXDRec . CycTo = 0 ;
  RS232TXDRec . CycFr = 0 ;
  RS232TXDRec . CycTo = 0 ;
  RS232TXDRec . State = IdleState ;
  RS485RXDRec . CycFr = 0 ;
  RS485RXDRec . CycTo = 0 ;
  RS485TXDRec . CycFr = 0 ;
  RS485TXDRec . CycTo = 0 ;
  RS485TXDRec . State = IdleState ;
  TimerCounter = 0 ;
  TimeEvent = false ;
  RS232CommandIndex = 0 ;
  RS485CommandIndex = 0 ;
  ResetOperation ( ) ;
} /* SoftwareInit */


int main ( void )
{
  /*----------------------------------------------------------------*/
  HardwareInit ( Speed38400bps , Speed9600bps ) ;
  EnvirInit ( ) ;
  SoftwareInit ( ) ;
  UART485ReceiveMode ( ) ;
  wdt_enable ( WDTO_1S ) ;
  sei ( ) ;
  UART232SendSerialFlashTxt ( ( uint16_t ) DeviceName ) ;
  UART232SendSerial ( 0x0D ) ;
  UART232SendSerial ( 0x0A ) ;
  for ( ; ; )
  {
    wdt_reset ( ) ;
    if ( TimeEvent )
    {
      TimeEvent = false ;
      LEDPort ^= ( 1 << StatusLedPosInPort ) ;
    } /* if */ ;
    if ( UART232DataPresent ( ) )
      UART232ProcessData ( UART232GetSerial ( ) ) ;
    if ( SerialService . SerialService )
      SerialService . SerialService ( ) ;
  } /* for */ ;
  return ( 0 ) ;
} /* main */


